let name = {{ grid_id | tojson }};
if (typeof mols2grid_lists !== "undefined") {
    var listObj = mols2grid_lists[name];
} else if (typeof window.parent.mols2grid_lists !== "undefined") {
    var listObj = window.parent.mols2grid_lists[name];
}
if (typeof listObj !== "undefined") {
    var mask = {{ mask }};
    listObj.filter(function (item) {
        return mask[item.values()["mols2grid-id"]];
    });
}
